/*
** Module   : ClientBuilderOptions.java
** Abstract : store the parameters read from directory.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131128 First version.
*/

package com.goldencode.p2j.main;

import java.util.*;

import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.swing.*;
import com.goldencode.p2j.util.*;

/**
 * The parameters stored inside this structure are get from directory on server start-up 
 * and are used as default parameters to create and spawn a web client process.
 * 
 * @author mag
 *
 */
public class ClientBuilderOptions
{   
   /** Directory node name */
   private static final String DIRECTORY_NODE_ID = "chuiWeb/";

   /** Pool size */
   private static final int POOL_SIZE = 8;

   /** Timeout */
   private static final int TIMEOUT = 30000;
   
   /** Default command */ 
   private static final String COMMAND = "java";
   
   /** Default CLASSPATH */ 
   private static final String CLASSPATH = "./p2j.jar";

   /** Default native library path */
   private static final String LIBPATH = "./";

   /** Default spawner location */
   private static final String SPAWNER = "./spawn";

   /** Default working directory */
   private static final String WORKING_DIRECTORY = "./";

   /** Default client configuration file */
   private static final String CONFIGFILE = null;
   
   /** Default JVM arguments */
   private static final String JVM_ARGUMENTS = 
            "-Xmx512m -XX:MaxPermSize=64m -Djava.awt.headless=true";
   
   /** Pool size */
   private static int poolSize;

   /** Timeout */
   private static long timeout;
   
   /** Secured flag */
   private static boolean secure;
   
   /** Command */
   private static String command;

   /** Class path */
   private static String classPath;
   
   /** Native library path */
   private static String libPath;

   /** User working directory */
   private static String workingDirectory;

   /** Client configuration file */
   private static String configFile;

   /** Spawner */
   private static String spawner;
   
   /** JVM arguments */
   private static List<String> jvmArguments = new LinkedList<String>();

   /** Terminal option **/
   private static Map<String, String> terminalOptions = new HashMap<>();
   
   /** Main class, must have a main method */
   private static Class<?> mainClass = ClientDriver.class;
   
   /** Singleton instance */
   private static ClientBuilderOptions instance;

   /**
    * Initialize. Read all configuration parameters from directory.
    */
   static
   {
      instance = new ClientBuilderOptions();
      // Pool size
      poolSize = getNode("poolSize", POOL_SIZE);
      // Pool size
      timeout = getNode("timeout", TIMEOUT);
      // Secure
      secure = getNode("secure", true);
      // Command
      command = quotedString(getNode("command", COMMAND));
      // Class path
      classPath = quotedString(getNode("classpath", CLASSPATH));
      // Native library path
      libPath = quotedString(getNode("libPath", LIBPATH));
      // Spawner
      spawner = quotedString(getNode("spawner", SPAWNER)); 
      // Working directory
      workingDirectory = quotedString(getNode("workingDir", WORKING_DIRECTORY)); 
      // Configuration file
      configFile = quotedString(getNode("configFile", CONFIGFILE));
      // JVM arguments
      String jvm = getNode("jvmArgs", JVM_ARGUMENTS);
      jvmArguments.addAll(Arrays.asList(jvm.split(" ")));            
      // Read Terminal Options
      readTerminalOptions();
   }

   /**
    * Private constructor.
    */
   private ClientBuilderOptions()
   {
      // hide constructor.
   }
   
   /**
    * Get ClientBuilderOptions singleton instance.
    */
   public static ClientBuilderOptions getInstance()
   {
      return instance;
   }
   
   /**
    * When the string contains space characters a quoted string is build.
    * On Windows OS spaces could be used in file path, file name, font name and other
    * strings. In order to use such kind of strings in API calls the string should be
    * transformed in quoted form. 
    * 
    * @param   text  Text to search for spaces.
    * 
    * @return  If no spaces found return the initial string otherwise return a quoted string. 
    */
   private static String quotedString(String text)
   {
      if (text.startsWith("\"") && text.endsWith("\""))
      {
         return text;
      }
      return (text.indexOf(" ") > -1) ? "\\\"" + text +"\\\"" : text;
   }
   
  /**
    * Read terminal options.
    */
   private static void readTerminalOptions()
   {
      // default terminal options
      int rows = TerminalOptions.DEF_ROWS;
      int columns = TerminalOptions.DEF_COLUMNS;
      String background = "0x" + Integer.toHexString(TerminalOptions.DEF_BACKGROUND.getRGB());
      String foreground = "0x" + Integer.toHexString(TerminalOptions.DEF_FOREGROUND.getRGB());
      String selection = "0x" + Integer.toHexString(TerminalOptions.DEF_SELECTION.getRGB());
      String fontname = TerminalOptions.DEF_FONTNAME;
      int fontsize = TerminalOptions.DEF_FONTSIZE;

      // get registered terminal options
      rows = getNode("rows", rows);
      columns = getNode("columns", columns);
      background = getNode("background", background);
      foreground = getNode("foreground", foreground);
      selection = getNode("selection", selection);
      fontname = quotedString(getNode("fontname", fontname));
      fontsize = getNode("fontsize", fontsize);
      
      // build terminal options map
      terminalOptions.put("type", OutputManager.CHUI_DRIVER_WEB);
      terminalOptions.put("rows", String.valueOf(rows));
      terminalOptions.put("columns", String.valueOf(columns));
      terminalOptions.put("background", background);
      terminalOptions.put("foreground", foreground);
      terminalOptions.put("fontname", fontname);
      terminalOptions.put("fontsize", String.valueOf(fontsize));      
   }   
    
   /**
    * Get a string value from a directory node.
    *  
    * @param   node
    *          Node name.
    * @param   defa
    *          Default value if node not found.
    *          
    * @return  Node value if exists or the default value otherwise.
    */
   private static String getNode(String node, String defa)
   {
      return Utils.getDirectoryNodeString(null, 
                                          ClientBuilderOptions.DIRECTORY_NODE_ID + node, 
                                          defa, 
                                          false);      
   }

   /**
    * Get an integer value from a directory node.
    *  
    * @param   node
    *          Node name.
    * @param   defa
    *          Default value if node not found.
    *          
    * @return  Node value if exists or the default value otherwise.
    */
   private static int getNode(String node, int defa)
   {
      return Utils.getDirectoryNodeInt(null, 
                                       ClientBuilderOptions.DIRECTORY_NODE_ID + node, 
                                       defa, 
                                       false);      
   }
   
   /**
    * Get a boolean value from a directory node.
    *  
    * @param   node
    *          Node name.
    * @param   defa
    *          Default value if node not found.
    *          
    * @return  Node value if exists or the default value otherwise.
    */
   private static boolean getNode(String node, boolean defa)
   {
      return Utils.getDirectoryNodeBoolean(null, 
                                           ClientBuilderOptions.DIRECTORY_NODE_ID + node, 
                                           defa, 
                                           false);      
   }
   
   /**
    * Get the security mode..
    * 
    * @return  <code>true</code> use a secured connection <code>false</code> otherwise.
    */
   public boolean isSecure()
   {
      return ClientBuilderOptions.secure;
   }

   /**
    * Get command
    * 
    * @return  Command name string.
    */
   public String getCommand()
   {
      return ClientBuilderOptions.command;
   }
   
   /**
    * Get classpath
    * 
    * @return  Class path string.
    */
   public String getClassPath()
   {
      return ClientBuilderOptions.classPath;
   }

   /**
    * Get native library path.
    * 
    * @return  Native library path string.
    */
   public String getLibPath()
   {
      return ClientBuilderOptions.libPath;
   }

   /**
    * Get user working directory.
    * 
    * @return  User working directory.
    */
   public String getWorkingDirectory()
   {
      return ClientBuilderOptions.workingDirectory;
   }

   /**
    * Get configuration file name.
    * 
    * @return  Configuration file name.
    */
   public String getConfigFile()
   {
      return ClientBuilderOptions.configFile;
   }
   
   /**
    * Get JVM arguments
    * 
    * @return  A List with JVM arguments.
    */
   public List<String> getJvmArguments()
   {
      return ClientBuilderOptions.jvmArguments;
   }

   /**
    * Get spawner command.
    * 
    * @return  The spawner command;
    */
   public String getSpawner()
   {
      return ClientBuilderOptions.spawner;
   }

   /**
    * Get Terminal Options.
    * 
    * @return  The spawner command;
    */
   public Map<String, String> getTerminalOptions()
   {
      return ClientBuilderOptions.terminalOptions;
   }

   /**
    * Get the Java process main class.
    * 
    * @return  The main class.
    */
   public Class<?> getMainClass()
   {
      return ClientBuilderOptions.mainClass;
   }

   /**
    * Get temporary account pool size.
    * 
    * @return  Temporary account pool size.
    */
   public int getPoolSize()
   {
      return ClientBuilderOptions.poolSize;
   }   

   /**
    * Get timeout.
    * 
    * @return  Timeout.
    */
   public long getTimeout()
   {
      return ClientBuilderOptions.timeout;
   }   
}
